@@ -1,5 +1,6 @@
 #!/usr/bin/env python3
 import os
+from pathlib import Path
 import shutil
 import zipfile
 
@@ -8,11 +9,14 @@ def main():
     py_bin = shutil.which("python3")
     if not py_bin:
         py_bin = shutil.which("python")
+    if not py_bin:
+        raise RuntimeError("Cannot find path for python")
 
-    dist_dir = os.path.join(os.path.split(os.path.abspath(__file__))[0], "../dist")
+    proj_dir = Path(Path(__file__).parent, "../").resolve()
+    dist_dir = Path(proj_dir, "dist")
 
     shutil.rmtree(dist_dir, ignore_errors=True)
-    os.chdir(os.path.join(os.path.split(os.path.abspath(__file__))[0], "../"))
+    os.chdir(proj_dir)
     os.system(py_bin + " -m build .")
 
     for zip_file in os.listdir(dist_dir):
