@@ -24,7 +24,7 @@
 namespace nb = nanobind;
 using namespace nb::literals;
 
-std::map<int, int> rowbytesMap = {
+std::map<int, size_t> rowbytesMap = {
     { 0, 1 }, // Grayscale
     { 2, 3 }, // RGB
     { 3, 1 }, // Palette
@@ -37,41 +37,78 @@ NB_MODULE(MODULE_NAME, m) {
     m.attr("__version__") = VERSION_INFO;
 
     m.def("create_frame_from_rgb", [](
-        nb::ndarray<unsigned char, nb::shape<nb::any>> *pixels,
+        nb::ndarray<unsigned char, nb::shape<nb::any, nb::any, 3>> *pixels,
         unsigned int width, unsigned int height,
-        nb::ndarray<unsigned char, nb::shape<nb::any>> *trns_color = NULL,
         unsigned delayNum = apngasm::DEFAULT_FRAME_NUMERATOR,
         unsigned delayDen = apngasm::DEFAULT_FRAME_DENOMINATOR
     ) APNGASM_PY_DECLSPEC {
-        apngasm::rgb *pixelsNew = new apngasm::rgb[pixels->shape(0)];
+        apngasm::rgb *pixelsNew = new apngasm::rgb[pixels->shape(0)*pixels->shape(1)];
         unsigned char *pixels_ptr = pixels->data();
-        for (int i = 0; i < pixels->shape(0) / 3; ++i) {
-            pixelsNew[i].r = pixels_ptr[3 * i];
-            pixelsNew[i].g = pixels_ptr[3 * i + 1];
-            pixelsNew[i].b = pixels_ptr[3 * i + 2];
+
+        for (int y = 0; y < pixels->shape(0); ++y) {
+            for (int x = 0; x < pixels->shape(1); ++x) {
+                pixelsNew[pixels->shape(1)*y + x].r = pixels_ptr[pixels->shape(2)*pixels->shape(1)*y + pixels->shape(2)*x];
+                pixelsNew[pixels->shape(1)*y + x].g = pixels_ptr[pixels->shape(2)*pixels->shape(1)*y + pixels->shape(2)*x + 1];
+                pixelsNew[pixels->shape(1)*y + x].b = pixels_ptr[pixels->shape(2)*pixels->shape(1)*y + pixels->shape(2)*x + 2];
+            }
         }
 
-        apngasm::rgb *trns_colorNew = new apngasm::rgb[trns_color->shape(0)];
-        unsigned char *trns_color_ptr = trns_color->data();
-        for (int i = 0; i < trns_color->shape(0) / 3; ++i) {
-            trns_colorNew[i].r = trns_color_ptr[3 * i];
-            trns_colorNew[i].g = trns_color_ptr[3 * i + 1];
-            trns_colorNew[i].b = trns_color_ptr[3 * i + 2];
-        }
-
-        const apngasm::APNGFrame &frame = apngasm::APNGFrame(pixelsNew, width, height, trns_colorNew, delayNum, delayDen);
+        const apngasm::APNGFrame frame(pixelsNew, width, height, NULL, delayNum, delayDen);
         delete[] pixelsNew;
-        delete[] trns_colorNew;
-        return nb::cast(frame);
+        return frame;
     },
-    "pixels"_a, "width"_a, "height"_a, "trns_color"_a = NULL, "delay_num"_a = apngasm::DEFAULT_FRAME_NUMERATOR, "delay_den"_a = apngasm::DEFAULT_FRAME_DENOMINATOR,
+    "pixels"_a, "width"_a, "height"_a, "delay_num"_a = apngasm::DEFAULT_FRAME_NUMERATOR, "delay_den"_a = apngasm::DEFAULT_FRAME_DENOMINATOR,
     R"pbdoc(
     Creates an APNGFrame from a bitmapped array of RBG pixel data.
     
-    :param numpy.ndarray pixels: The RGB pixel data, expressed as 1D numpy array.
+    :param numpy.typing.NDArray pixels: The RGB pixel data, expressed as 3D numpy array.
     :param int width: The width of the pixel data.
     :param int height: The height of the pixel data.
-    :param numpy.ndarray trns_color: An array of transparency data, expressed as 1D numpy array.
+    :param int delay_num: The delay numerator for this frame (defaults to DEFAULT_FRAME_NUMERATOR).
+    :param int delay_den: The delay denominator for this frame (defaults to DEFAULT_FRAME_DENMINATOR).
+    
+    :return: A APNGFrame object.
+    :rtype: apngasm_python._apngasm_python.APNGFrame
+    )pbdoc");
+
+    m.def("create_frame_from_rgb_trns", [](
+        nb::ndarray<unsigned char, nb::shape<nb::any, nb::any, 3>> *pixels,
+        unsigned int width, unsigned int height,
+        nb::ndarray<unsigned char, nb::shape<3>> *trns_color,
+        unsigned delayNum = apngasm::DEFAULT_FRAME_NUMERATOR,
+        unsigned delayDen = apngasm::DEFAULT_FRAME_DENOMINATOR
+    ) APNGASM_PY_DECLSPEC {
+        apngasm::rgb *pixelsNew = new apngasm::rgb[pixels->shape(0)*pixels->shape(1)];
+        unsigned char *pixels_ptr = pixels->data();
+
+        for (int y = 0; y < pixels->shape(0); ++y) {
+            for (int x = 0; x < pixels->shape(1); ++x) {
+                pixelsNew[pixels->shape(1)*y + x].r = pixels_ptr[pixels->shape(2)*pixels->shape(1)*y + pixels->shape(2)*x];
+                pixelsNew[pixels->shape(1)*y + x].g = pixels_ptr[pixels->shape(2)*pixels->shape(1)*y + pixels->shape(2)*x + 1];
+                pixelsNew[pixels->shape(1)*y + x].b = pixels_ptr[pixels->shape(2)*pixels->shape(1)*y + pixels->shape(2)*x + 2];
+            }
+        }
+
+        apngasm::rgb *trns_colorNew = new apngasm::rgb;
+
+        unsigned char *trns_color_ptr = trns_color->data();
+        trns_colorNew->r = trns_color_ptr[0];
+        trns_colorNew->g = trns_color_ptr[1];
+        trns_colorNew->b = trns_color_ptr[2];
+
+        const apngasm::APNGFrame frame(pixelsNew, width, height, trns_colorNew, delayNum, delayDen);
+        delete[] pixelsNew;
+        delete trns_colorNew;
+        return frame;
+    },
+    "pixels"_a, "width"_a, "height"_a, "trns_color"_a, "delay_num"_a = apngasm::DEFAULT_FRAME_NUMERATOR, "delay_den"_a = apngasm::DEFAULT_FRAME_DENOMINATOR,
+    R"pbdoc(
+    Creates an APNGFrame from a bitmapped array of RBG pixel data, with one color treated as transparent.
+    
+    :param numpy.typing.NDArray pixels: The RGB pixel data, expressed as 3D numpy array.
+    :param int width: The width of the pixel data.
+    :param int height: The height of the pixel data.
+    :param numpy.typing.NDArray trns_color: The color [r, g, b] to be treated as transparent, expressed as 1D numpy array.
     :param int delay_num: The delay numerator for this frame (defaults to DEFAULT_FRAME_NUMERATOR).
     :param int delay_den: The delay denominator for this frame (defaults to DEFAULT_FRAME_DENMINATOR).
     
@@ -80,30 +117,32 @@ NB_MODULE(MODULE_NAME, m) {
     )pbdoc");
     
     m.def("create_frame_from_rgba", [](
-        nb::ndarray<unsigned char, nb::shape<nb::any>> *pixels,
+        nb::ndarray<unsigned char, nb::shape<nb::any, nb::any, 4>> *pixels,
         unsigned int width, unsigned int height,
         unsigned delayNum = apngasm::DEFAULT_FRAME_NUMERATOR,
         unsigned delayDen = apngasm::DEFAULT_FRAME_DENOMINATOR
     ) APNGASM_PY_DECLSPEC {
-        apngasm::rgba *pixelsNew = new apngasm::rgba[pixels->shape(0)];
+        apngasm::rgba *pixelsNew = new apngasm::rgba[pixels->shape(0)*pixels->shape(1)];
         unsigned char *pixels_ptr = pixels->data();
 
-        for (int i = 0; i < pixels->shape(0) / 4; ++i) {
-            pixelsNew[i].r = pixels_ptr[4 * i];
-            pixelsNew[i].g = pixels_ptr[4 * i + 1];
-            pixelsNew[i].b = pixels_ptr[4 * i + 2];
-            pixelsNew[i].a = pixels_ptr[4 * i + 3];
+        for (int y = 0; y < pixels->shape(0); ++y) {
+            for (int x = 0; x < pixels->shape(1); ++x) {
+                pixelsNew[pixels->shape(1)*y + x].r = pixels_ptr[pixels->shape(2)*pixels->shape(1)*y + pixels->shape(2)*x];
+                pixelsNew[pixels->shape(1)*y + x].g = pixels_ptr[pixels->shape(2)*pixels->shape(1)*y + pixels->shape(2)*x + 1];
+                pixelsNew[pixels->shape(1)*y + x].b = pixels_ptr[pixels->shape(2)*pixels->shape(1)*y + pixels->shape(2)*x + 2];
+                pixelsNew[pixels->shape(1)*y + x].a = pixels_ptr[pixels->shape(2)*pixels->shape(1)*y + pixels->shape(2)*x + 3];
+            }
         }
 
         const apngasm::APNGFrame frame(pixelsNew, width, height, delayNum, delayDen);
         delete[] pixelsNew;
-        return nb::cast(frame);
+        return frame;
     },
     "pixels"_a, "width"_a, "height"_a, "delay_num"_a = apngasm::DEFAULT_FRAME_NUMERATOR, "delay_den"_a = apngasm::DEFAULT_FRAME_DENOMINATOR,
     R"pbdoc(
     Creates an APNGFrame from a bitmapped array of RBGA pixel data.
     
-    :param numpy.ndarray pixels: The RGBA pixel data, expressed as 1D numpy array.
+    :param numpy.typing.NDArray pixels: The RGBA pixel data, expressed as 3D numpy array.
     :param int width: The width of the pixel data.
     :param int height: The height of the pixel data.
     :param int delay_num: The delay numerator for this frame (defaults to DEFAULT_FRAME_NUMERATOR)
@@ -178,10 +217,10 @@ NB_MODULE(MODULE_NAME, m) {
         then set frame.width, frame.height, frame.color_type, frame.pixels,
         frame.palette, frame.delay_num, frame.delay_den manually.
         
-        :param pixels: The RGB pixel data.
+        :param apngasm_python._apngasm_python.rgb pixels: The RGB pixel data.
         :param int width: The width of the pixel data.
         :param int height: The height of the pixel data.
-        :param trns_color: An array of transparency data.
+        :param apngasm_python._apngasm_python.rgb trns_color: The color [r, g, b] to be treated as transparent.
         :param int delay_num: The delay numerator for this frame (defaults to DEFAULT_FRAME_NUMERATOR).
         :param int delay_den: The delay denominator for this frame (defaults to DEFAULT_FRAME_DENMINATOR).
         )pbdoc")
@@ -196,7 +235,7 @@ NB_MODULE(MODULE_NAME, m) {
         then set frame.width, frame.height, frame.color_type, frame.pixels,
         frame.palette, frame.delay_num, frame.delay_den manually.
         
-        :param pixels: The RGBA pixel data.
+        :param apngasm_python._apngasm_python.rgba pixels: The RGBA pixel data.
         :param int width: The width of the pixel data.
         :param int height: The height of the pixel data.
         :param int delay_num: The delay numerator for this frame (defaults to DEFAULT_FRAME_NUMERATOR).
@@ -216,26 +255,27 @@ NB_MODULE(MODULE_NAME, m) {
 
         .def_prop_rw("pixels", 
         [](apngasm::APNGFrame &t) APNGASM_PY_DECLSPEC {
-            int rowbytes = rowbytesMap[t._colorType];
-            size_t shape[1] = { t._height * t._width * rowbytes };
-            return nb::cast(nb::ndarray<nb::numpy, unsigned char, nb::shape<nb::any>>(t._pixels, 1, shape));
+            size_t rowbytes = rowbytesMap[t._colorType];
+            size_t shape[3] = { t._height, t._width, rowbytes };
+            return nb::ndarray<nb::numpy, unsigned char, nb::shape<nb::any, nb::any, nb::any>>(t._pixels, 3, shape);
         },
-        [](apngasm::APNGFrame &t, nb::ndarray<unsigned char, nb::shape<nb::any>> *v) APNGASM_PY_DECLSPEC {
-            int rowbytes = rowbytesMap[t._colorType];
-            unsigned char *pixelsNew = new unsigned char[v->shape(0)];
+        [](apngasm::APNGFrame &t, nb::ndarray<unsigned char, nb::shape<nb::any, nb::any, nb::any>> *v) APNGASM_PY_DECLSPEC {
+            size_t rowbytes = rowbytesMap[t._colorType];
+            unsigned char *pixelsNew = new unsigned char[v->size()];
             unsigned char *v_ptr = v->data();
-            for (int i = 0; i < v->shape(0); ++i) {
-                pixelsNew[i] = *v_ptr;
-                ++v_ptr;
+
+            for (int i = 0; i < v->size(); ++i) {
+                pixelsNew[i] = v_ptr[i];
             }
             t._pixels = pixelsNew;
 
             t._rows = new png_bytep[t._height * sizeof(png_bytep)];
-            for (int j = 0; j < t._height; ++j)
+            for (int j = 0; j < t._height; ++j) {
                 t._rows[j] = t._pixels + j * rowbytes;
+            }
         },
         R"pbdoc(
-        The raw pixel data of frame, expressed as a 1D numpy array in Python.
+        The raw pixel data of frame, expressed as a 3D numpy array in Python.
         Note that setting this value will also set the variable 'rows' internally.
         This should be set AFTER you set the width, height and color_type.
         )pbdoc")
@@ -276,7 +316,7 @@ NB_MODULE(MODULE_NAME, m) {
                 paletteView[i][2] = t._palette[i].b;
             }
             size_t shape[2] = { 256, 3 };
-            return nb::cast(nb::ndarray<nb::numpy, unsigned char, nb::shape<256, 3>>(paletteView, 2, shape)); 
+            return nb::ndarray<nb::numpy, unsigned char, nb::shape<256, 3>>(paletteView, 2, shape); 
         },
         [](apngasm::APNGFrame &t, nb::ndarray<unsigned char, nb::shape<256, 3>> *v) APNGASM_PY_DECLSPEC {
             unsigned char *v_ptr = v->data();
@@ -288,14 +328,14 @@ NB_MODULE(MODULE_NAME, m) {
             }
         },
         R"pbdoc(
-        The palette data of frame. Only applies to 'P' mode Image (i.e. Not RGB, RGBA)
-        Expressed as 2D numpy array in format of [[r0, g0, b0], [r1, g1, b1], ..., [r255, g255, b255]] in Python
+        The palette data of frame. Only applies to 'P' mode Image (i.e. Not RGB, RGBA).
+        Expressed as 2D numpy array in format of [[r0, g0, b0], [r1, g1, b1], ..., [r255, g255, b255]] in Python.
         )pbdoc")
 
         .def_prop_rw("transparency", 
         [](apngasm::APNGFrame &t) APNGASM_PY_DECLSPEC {
             size_t shape[1] = { static_cast<size_t>(t._transparencySize) };
-            return nb::cast(nb::ndarray<nb::numpy, unsigned char, nb::shape<nb::any>>(t._transparency, 1, shape)); 
+            return nb::ndarray<nb::numpy, unsigned char, nb::shape<nb::any>>(t._transparency, 1, shape); 
         },
         [](apngasm::APNGFrame &t, nb::ndarray<unsigned char, nb::shape<nb::any>> *v) APNGASM_PY_DECLSPEC {
             unsigned char *v_ptr = v->data();
@@ -305,7 +345,8 @@ NB_MODULE(MODULE_NAME, m) {
             }
         },
         R"pbdoc(
-        The transparency data of frame. Expressed as 1D numpy array.
+        The transparency color of frame that is treated as transparent, expressed as 1D numpy array.
+        For more info, refer to 'tRNS Transparency' in http://www.libpng.org/pub/png/spec/1.2/PNG-Chunks.html
         )pbdoc")
 
         .def_prop_rw("palette_size", 
@@ -353,7 +394,7 @@ NB_MODULE(MODULE_NAME, m) {
         R"pbdoc(
         Construct APNGAsm object from an existing vector of apngasm frames.
     
-        :param list frames: A list of APNGFrame objects.
+        :param list[apngasm_python._apngasm_python.APNGFrame] frames: A list of APNGFrame objects.
         )pbdoc")
 
         .def("add_frame", nb::overload_cast<const apngasm::APNGFrame &>(&apngasm::APNGAsm::addFrame),
@@ -361,7 +402,7 @@ NB_MODULE(MODULE_NAME, m) {
         R"pbdoc(
         Adds an APNGFrame object to the frame vector.
     
-        :param frame: The APNGFrame object to be added
+        :param frame: The APNGFrame object to be added.
         :type frame: apngasm_python._apngasm_python.APNGFrame
 
         :return: The new number of frames/the number of this frame on the frame vector.
@@ -391,10 +432,10 @@ NB_MODULE(MODULE_NAME, m) {
         then set frame.width, frame.height, frame.color_type, frame.pixels,
         frame.palette, frame.delay_num, frame.delay_den manually.
         
-        :param pixels_rgb: The RGB pixel data.
+        :param apngasm_python._apngasm_python.rgb pixels_rgb: The RGB pixel data.
         :param int width: The width of the pixel data.
         :param int height: The height of the pixel data.
-        :param trns_color: An array of transparency data.
+        :param apngasm_python._apngasm_python.rgb trns_color: The color [r, g, b] to be treated as transparent.
         :param int delay_num: The delay numerator for this frame (defaults to DEFAULT_FRAME_NUMERATOR).
         :param int delay_den: The delay denominator for this frame (defaults to DEFAULT_FRAME_DENMINATOR).
 
@@ -412,7 +453,7 @@ NB_MODULE(MODULE_NAME, m) {
         then set frame.width, frame.height, frame.color_type, frame.pixels,
         frame.palette, frame.delay_num, frame.delay_den manually.
         
-        :param pixels_rgba: The RGBA pixel data.
+        :param apngasm_python._apngasm_python.rgba pixels_rgba: The RGBA pixel data.
         :param int width: The width of the pixel data.
         :param int height: The height of the pixel data.
         :param int delay_num: The delay numerator for this frame (defaults to DEFAULT_FRAME_NUMERATOR).
@@ -441,7 +482,7 @@ NB_MODULE(MODULE_NAME, m) {
         :param str file_path: The file path to the PNG image to be disassembled.
 
         :return: A vector containing the frames of the disassembled PNG.
-        :rtype: list
+        :rtype: list[apngasm_python._apngasm_python.APNGFrame]
         )pbdoc")
 
         .def("save_pngs", &apngasm::APNGAsm::savePNGs,
@@ -463,10 +504,10 @@ NB_MODULE(MODULE_NAME, m) {
         For more details on animation specs see:
         https://github.com/Genshin/PhantomStandards
         
-        :param str file_path: The path of JSON or XML file
+        :param str file_path: The path of JSON or XML file.
 
         :return: A vector containing the frames
-        :rtype: list
+        :rtype: list[apngasm_python._apngasm_python.APNGFrame]
         )pbdoc")
 
         .def("save_json", &apngasm::APNGAsm::saveJSON,
@@ -493,12 +534,12 @@ NB_MODULE(MODULE_NAME, m) {
         :rtype: bool
         )pbdoc")
 
-        .def("set_apng_asm_listener", &apngasm::APNGAsm::setAPNGAsmListener,
-        "listener"_a = NULL,
+        .def("set_apngasm_listener", &apngasm::APNGAsm::setAPNGAsmListener,
+        "listener"_a = nb::none(),
         R"pbdoc(
         Sets a listener.
         
-        :param apngasm_python._apngasm_python.IAPNGAsmListener listener: A pointer to the listener object. If the argument is NULL a default APNGAsmListener will be created and assigned.8
+        :param Optional[apngasm_python._apngasm_python.IAPNGAsmListener] listener: A pointer to the listener object. If the argument is NULL a default APNGAsmListener will be created and assigned.
         )pbdoc")
 
         .def("set_loops", &apngasm::APNGAsm::setLoops,
@@ -521,15 +562,15 @@ NB_MODULE(MODULE_NAME, m) {
         R"pbdoc(
         Returns the frame vector.
 
-        :return: frame vector
-        :rtype: numpy.ndarray
+        :return: frame vector.
+        :rtype: list[apngasm_python._apngasm_python.APNGFrame]
         )pbdoc")
 
         .def("get_loops", &apngasm::APNGAsm::getLoops,
         R"pbdoc(
         Returns the loop count.
         
-        :return: loop count
+        :return: loop count.
         :rtype: int
         )pbdoc")
 
@@ -537,7 +578,7 @@ NB_MODULE(MODULE_NAME, m) {
         R"pbdoc(
         Returns the flag of skip first frame.
         
-        :return: flag of skip first frame
+        :return: flag of skip first frame.
         :rtype: bool
         )pbdoc")
 
@@ -545,7 +586,7 @@ NB_MODULE(MODULE_NAME, m) {
         R"pbdoc(
         Returns the number of frames.
         
-        :return: number of frames
+        :return: number of frames.
         :rtype: int
         )pbdoc")
 
@@ -555,7 +596,7 @@ NB_MODULE(MODULE_NAME, m) {
         Leaves the apngasm object in a clean state.
         Returns number of frames disposed of.
         
-        :return: number of frames disposed of
+        :return: number of frames disposed of.
         :rtype: int
         )pbdoc")
 
@@ -563,7 +604,7 @@ NB_MODULE(MODULE_NAME, m) {
         R"pbdoc(
         Returns the version of APNGAsm.
         
-        :return: the version of APNGAsm
+        :return: the version of APNGAsm.
         :rtype: str
         )pbdoc")
 
